/*
 * $Id: AlarmMessage.cpp,v 1.2 2003/02/22 12:21:47 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * Modified by Anton Maslovsky <my-zaurus@narod.ru> 2003/01/28
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "AlarmMessage.h"
#include "qpealarmclock.h"
#include <qpushbutton.h>
#include <qtimer.h>
#include <qpe/sound.h>

// includes for alarm led
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "sharp_char.h"

AlarmMessage::AlarmMessage(AlarmDetails* pNewAlarm, QWidget* parent, const char* name, bool modal,
                           WFlags fl) :
                           AlarmMessageBase(parent, name, modal, fl)
{
    pAlarmClock = (AlarmClock *)parent;
    pAlarm = pNewAlarm;

    connect(btnSnooze, SIGNAL(clicked()), this, SLOT(snoozeClicked()));

    if (!pAlarm->message().isNull())
      lblALARM->setText(pAlarm->message());

    int hour, minute;
    hour = pAlarm->time().hour();
    minute = pAlarm->time().minute();
    if (pAlarmClock->twelveHourClock)
    {
      if (hour < 12)
        lblTime->setText(QString().sprintf("%d:%02d",
          hour == 0 ? 12 : hour, minute) + tr(" AM"));
      else
      lblTime->setText(QString().sprintf("%d:%02d",
          hour == 12 ? 12 : hour - 12, minute) + tr(" PM"));
    }
    else
      lblTime->setText(QString().sprintf("%02d:%02d", hour, minute));

    btnSnooze->setText(tr("Snooze (") + QString::number(pAlarm->snoozeInterval()) + " min)");  

    // sound alarm if loud notification is set
    if (pAlarm->notification() == AlarmDetails::loud)
    {  
      stop = 0;
      timer = new QTimer(this, "timer");
      timer->start(2000);
    
      connect(timer, SIGNAL(timeout()), this, SLOT(timerTriggered()));
    
      Sound::soundAlarm();
    }

    // blink led
    blinkLed(pAlarm->led());
}

AlarmMessage::~AlarmMessage()
{
  // reset led to default
  blinkLed(LED_COLLIE_1_DEFAULT);
}

void AlarmMessage::blinkLed(int led)
{
  int fd;
  sharp_led_status st;

  fd = open ("/dev/sharp_led", O_WRONLY);
  st.which = SHARP_LED_COLLIE_1;
  st.status = led;
  ioctl(fd, SHARP_LED_SETSTATUS, &st);
}

void AlarmMessage::timerTriggered()
{
    if ( stop < 20 ) 
    {
      Sound::soundAlarm();
      stop++;
    } 
    else 
    {
      stop = 0;
      timer->stop();
      //delete timer;
    }
}

void AlarmMessage::snoozeClicked()
{
  qDebug("Snoozing alarm: " + pAlarm->toString());

  // reschedule another alarm in snoozeInterval() minuties
  AlarmDetails* pSnoozeAlarm = pAlarmClock->snoozeAlarm();

  pSnoozeAlarm->setAlarm(true,
    QTime::currentTime().addSecs(60 * pAlarm->snoozeInterval()),
    AlarmDetails::once,
    pAlarm->message(),
    pAlarm->notification(),
    pAlarm->led(),
    pAlarm->snoozeInterval(),
    pAlarm->weekdaysFlags());

  AlarmMessageBase::accept();
}
