/*
 * $Id: AlarmSet.cpp,v 1.2 2003/02/22 12:21:47 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * Modified by Anton Maslovsky <my-zaurus@narod.ru> 2003/01/28
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "AlarmSet.h"

#include <qpe/timestring.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlineedit.h>

#include "qpealarmclock.h"
#include "sharp_char.h"

AlarmSet::AlarmSet(AlarmDetails* alarmDetailsInit, QWidget* parent, 
		   const char* name, bool modal, WFlags fl) : 
                   AlarmSetBase(parent, name, modal, fl), 
                   alarmDetails(alarmDetailsInit)
{
    for (int i = AlarmDetails::once; i <= AlarmDetails::custom; i++)
      cboFrequency->insertItem(AlarmDetails::frequencyLabel(static_cast<AlarmDetails::Frequency>(i)));

    connect(cboFrequency, SIGNAL(activated(int)), this, SLOT(cboFrequencyActivated(int)));
    cboFrequency->setCurrentItem(alarmDetails->frequency());

    // populate checkboxes
    chkWeekdays[0] = chkMon;
    chkWeekdays[1] = chkTue;
    chkWeekdays[2] = chkWed;
    chkWeekdays[3] = chkThu;
    chkWeekdays[4] = chkFri;
    chkWeekdays[5] = chkSat;
    chkWeekdays[6] = chkSun;

    setWeekdaysFlags(alarmDetails->weekdaysFlags(), alarmDetails->frequency() == AlarmDetails::custom);

    cboNotification->insertItem(AlarmDetails::notificationLabel(AlarmDetails::silent));
    cboNotification->insertItem(AlarmDetails::notificationLabel(AlarmDetails::loud));
    cboNotification->setCurrentItem(alarmDetails->notification());

    txtAlarmMessage->setText(alarmDetails->message());

    // always enable alarm when user starts to modify it
    chkAlarmEnabled->setChecked(true /* alarmDetails->enabled() */);

    for (int i = LED_COLLIE_1_DEFAULT; i <= LED_COLLIE_1_SOFTFLASH; i++)
      cboLed->insertItem(AlarmDetails::ledLabel(i));
    cboLed->setCurrentItem(alarmDetails->led());
    
    alarmTime = alarmDetails->time();
    
    spinMinute->setValue(alarmTime.minute());
    if (alarmTime.minute() < 10)
      spinMinute->setPrefix("0");
   
    if (((AlarmClock*)parent)->twelveHourClock)
    {
      spinHour->setMinValue(1);
      spinHour->setMaxValue(12);
      spinHour->setValue((alarmTime.hour() % 12 == 0) ? 12 : 
                         alarmTime.hour() < 12 ? alarmTime.hour() : 
                         alarmTime.hour() - 12);
      if (alarmTime.hour() >= 12)
        chkPM->setChecked(true);
    }
    else
    {
      spinHour->setMinValue(0);
      spinHour->setMaxValue(23);
      spinHour->setValue(alarmTime.hour());
      chkPM->hide();
    }

    connect(spinHour, SIGNAL(valueChanged(int)), this, 
            SLOT(hourChanged(int)));
    connect(spinMinute, SIGNAL(valueChanged(int)), this, 
            SLOT(minuteChanged(int)));
    connect(chkPM, SIGNAL(stateChanged(int)), this, 
            SLOT(chkPMChanged(int)));

    spinSnooze->setValue(alarmDetailsInit->snoozeInterval());
}

AlarmSet::~AlarmSet()
{
}

void AlarmSet::chkPMChanged(int state)
{
    if (((AlarmClock*)parent())->twelveHourClock)
    {
        int hour;
        if (state)
        {
            // PM
            hour = (spinHour->value() % 12 == 0) ? 12 : 
                    spinHour->value() + 12;
        }
        else
        {
            // AM
            hour = (spinHour->value() % 12 == 0) ? 0 : 
                    spinHour->value();
        }
        alarmTime = QTime(hour, alarmTime.minute());
    }
}

void AlarmSet::hourChanged(int value)
{
    int hour;
    if (((AlarmClock*)parent())->twelveHourClock)
    {
        if (chkPM->isChecked())
        {
            // PM
            hour = (value % 12 == 0) ? 12 : value + 12;
        }
        else
        {
            // AM
            hour = (value % 12 == 0) ? 0 : value;
        }
    }
    else
    {
        hour = value;
        if (value < 10)
            spinHour->setPrefix("0");
        else
            spinHour->setPrefix("");
    }

    alarmTime = QTime(hour, alarmTime.minute());
}

void AlarmSet::minuteChanged(int value)
{
    alarmTime = QTime(alarmTime.hour(), value);
    if (value < 10)
        spinMinute->setPrefix("0");
    else
        spinMinute->setPrefix("");
}

void AlarmSet::cboFrequencyActivated(int item)
{
  setWeekdaysFlags(AlarmDetails::weekdaysGroups[item], item == AlarmDetails::custom);
}

void AlarmSet::setWeekdaysFlags(AlarmDetails::Weekdays weekdays, bool enable /* = false */)
{
  for (int i = 0; i < NUM_WEEKDAYS; i++)
  {
    if (weekdays != AlarmDetails::WEEKDAYS_IGNORE)
      chkWeekdays[i]->setChecked(weekdays & AlarmDetails::flags[i]);

    chkWeekdays[i]->setEnabled(enable);
  }
}

AlarmDetails::Weekdays AlarmSet::weekdaysFlags() const
{
  unsigned result = AlarmDetails::WEEKDAYS_NONE;
  
  for (int i = 0; i < NUM_WEEKDAYS; i++)
  {
    if (chkWeekdays[i]->isChecked())
      result |= AlarmDetails::flags[i];
  }

  return static_cast<AlarmDetails::Weekdays>(result);
}

void AlarmSet::accept()
{
    alarmDetails->setAlarm(chkAlarmEnabled->isChecked(), alarmTime,
      static_cast<AlarmDetails::Frequency>(cboFrequency->currentItem()),
      txtAlarmMessage->text(),
      static_cast<AlarmDetails::Notification>(cboNotification->currentItem()),
      cboLed->currentItem(),
      spinSnooze->value(),
      weekdaysFlags());
      
    AlarmSetBase::accept();
}

