/*
 * $Id: qpealarmclock.cpp,v 1.2 2003/02/22 12:21:47 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * Modified by Anton Maslovsky <my-zaurus@narod.ru> 2003/01/28
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "qpealarmclock.h"
#include "AlarmSet.h"
#include "AlarmMessage.h"

#include <qtimer.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qpe/config.h>
#include <qtoolbutton.h>
#include <qpe/qpeapplication.h>
#include <qgroupbox.h>

AlarmClock::AlarmClock( QWidget* parent,  const char* name, WFlags fl )
    : AlarmClockBase( parent, name, fl )
{
    timer = new QTimer(this, "timer");
    timer->start(2000);
    
    connect(btnAbout, SIGNAL(clicked()), this, SLOT(aboutClicked()));
    connect(btnAlarm1, SIGNAL(clicked()), this, SLOT(alarm1clicked()));
    connect(btnAlarm2, SIGNAL(clicked()), this, SLOT(alarm2clicked()));
    connect(btnAlarm3, SIGNAL(clicked()), this, SLOT(alarm3clicked()));
    connect(btnAlarm4, SIGNAL(clicked()), this, SLOT(alarm4clicked()));
    connect(timer, SIGNAL(timeout()), this, SLOT(timerTriggered()));
    connect( qApp, SIGNAL(appMessage(const QCString&, const QByteArray&)),
	     this, SLOT(appMessage(const QCString&, const QByteArray&)) );
  
    for (int i = 0; i < NUM_ALARMS; i++)
      alarms[i] = new AlarmDetails(i);

    initTwelveHourClock();
    initUI();
}

AlarmClock::~AlarmClock()
{
    for (int i = 0; i < NUM_ALARMS; i++)
      delete alarms[i];

    delete timer;
}

void AlarmClock::timerTriggered()
{
    // Calculate time to next alarm
    QDateTime nextAlarm;
    QString strNextAlarm;
    for (int i = 0; i < NUM_ALARMS; i++)
    {
      if (alarms[i]->enabled() &&
          alarms[i]->nextDateTime().isValid() && (
          (!nextAlarm.isValid()) || (alarms[i]->nextDateTime() < nextAlarm)))
      {
        nextAlarm = alarms[i]->nextDateTime();
      }
    }

    if (nextAlarm.isValid())
    {
      int tot_secs = QDateTime::currentDateTime().secsTo(nextAlarm);
      int tot_mins = tot_secs / 60;
      int h = tot_mins / 60;
      int m = tot_mins % 60;
      strNextAlarm = tr("Next alarm in ");
      if (h == 1)
      {
        strNextAlarm += tr("1 hour, ");
      }
      else if (h > 1)
      {
        strNextAlarm += tr("%1 hours, ").arg(h);
      }

      if (m == 1)
      {
        strNextAlarm += tr("1 minute");  
      }
      else if (m == 0 && h == 0)
      {
        strNextAlarm += tr("less than 1 minute");
      }
      else
      {
        strNextAlarm += tr("%1 minutes").arg(m);
      }
    }
    else
    {
      strNextAlarm = tr("No alarms scheduled");
    }

    if (txtNextAlarm->text() != strNextAlarm)
    {
      txtNextAlarm->setText(strNextAlarm);
    }
}

void AlarmClock::aboutClicked()
{
    QMessageBox mb(tr("About Alarm Clock"),
         tr("<center><p>Version ") + VERSION +
         tr("<br><br>This program is Copyright (C) 2002 "
         "Dafydd&nbsp;Walters<br>"
         "&lt;dwalters@users.sourceforge.net&gt;<br>"
         "German translation by Peter Hildebrandt<br>"
         "Verison ") + VERSION + tr(" features by Anton Maslovsky<br>"
         "&lt;my-zaurus@narod.ru&gt;<br><br>"
         "This program may be modified or "
         "redistributed under the terms of "
         "the GNU&nbsp;General&nbsp;Public&nbsp;License<br>"
         "<b>http://www.gnu.org/licenses/gpl.html</b><br>"
         "The source code of this program is "
         "available from SourceForge at<br>"
         "<b>http://qpealarmclock.sourceforge.net</b></p></center>"),
         QMessageBox::NoIcon, QMessageBox::Ok, QMessageBox::NoButton,
         QMessageBox::NoButton, this);
    mb.showMaximized();
    mb.exec();
}

void AlarmClock::initUI()
{
    showAlarm(alarms[0], lblTime1, lblFrequency1, groupAlarm1);
    showAlarm(alarms[1], lblTime2, lblFrequency2, groupAlarm2);
    showAlarm(alarms[2], lblTime3, lblFrequency3, groupAlarm3);
    showAlarm(alarms[3], lblTime4, lblFrequency4, groupAlarm4);
}

void AlarmClock::showAlarm(AlarmDetails* alarmDetails, 
   QLabel* lblTime, QLabel* lblFrequency, QGroupBox* groupAlarm)
{
    groupAlarm->setTitle(buildAlarmText(alarmDetails));      

    if (twelveHourClock)
    {
      if (alarmDetails->time().hour() >= 12)
      {
        int hour = alarmDetails->time().hour() == 12 ? 12 :
                   alarmDetails->time().hour() - 12;
        lblTime->setText(QString::number(hour) + ":" +
                         QString().sprintf("%02d",
                         alarmDetails->time().minute()) +
                         tr(" PM"));
      }
      else
      {
        lblTime->setText(QString::number(alarmDetails->time().hour() == 0 ?
                         12 : alarmDetails->time().hour()) +
                         ":" + QString().sprintf("%02d",
                         alarmDetails->time().minute()) +
                         tr(" AM"));
      }
    }
    else
    {
      lblTime->setText(QString().sprintf("%02d:%02d",
                       alarmDetails->time().hour(),
                       alarmDetails->time().minute()));
    }

    lblFrequency->setText(AlarmDetails::frequencyLabel(alarmDetails->frequency()));
}

void AlarmClock::initTwelveHourClock() 
{
    Config config("qpe");
    config.setGroup("Time");
    twelveHourClock = config.readBoolEntry("AMPM", true);
}

QString AlarmClock::buildAlarmText(AlarmDetails* alarm) const
{
    QString title = alarm->message();
    const size_t MAX_LEN = alarm->enabled() ? 37 : 27;

    if(title.isNull())
      title = tr("Alarm");

    if (title.length() > MAX_LEN)
    {
      title.truncate(MAX_LEN);
      title.append("...");
    }
        
    if (!alarm->enabled())
       title += tr(" (disabled)");

    return title;

}

void AlarmClock::appMessage(const QCString& msg, const QByteArray& data)
{
    if (msg == "alarm(QDateTime,int)") 
    {
      QDataStream ds(data, IO_ReadOnly);
      int alarmNumber;
      QDateTime alarmDateTime;
      ds >> alarmDateTime >> alarmNumber;

      alarms[alarmNumber]->setAlarm(alarms[alarmNumber]->frequency() != AlarmDetails::once,
                                    alarms[alarmNumber]->time(),
                                    alarms[alarmNumber]->frequency(),
                                    alarms[alarmNumber]->message(),
                                    alarms[alarmNumber]->notification(),
                                    alarms[alarmNumber]->led(),
                                    alarms[alarmNumber]->snoozeInterval(),
                                    alarms[alarmNumber]->weekdaysFlags());
                                    
      AlarmMessage dlg(alarms[alarmNumber], this, "Alarm", true);
      dlg.showMaximized();
      initUI();
      dlg.exec();
    }
}

void AlarmClock::alarmClicked(AlarmDetails* alarm)
{
    AlarmSet dlg(alarm, this, 0, true);
    dlg.showMaximized();
    if (dlg.exec())
    {
      initUI();
    }
}

void AlarmClock::alarm1clicked()
{
    alarmClicked(alarms[0]);
}

void AlarmClock::alarm2clicked()
{
    alarmClicked(alarms[1]);
}

void AlarmClock::alarm3clicked()
{
    alarmClicked(alarms[2]);
}

void AlarmClock::alarm4clicked()
{
    alarmClicked(alarms[3]);
}
